/****
***** Copyright 2010 Intel Corporation All Rights Reserved.
*****
***** The source code, information and material contained herein are owned by Intel Corporation or its suppliers  *****
***** or licensors, and title to such Material remains with Intel Corporation or its suppliers or licensors.      *****
***** The Material contains proprietary information of Intel or its suppliers and licensors. The Material is      *****
***** protected by worldwide copyright laws and treaty provisions. No part of the Material may be used, copied,   *****
***** reproduced, modified, published, uploaded, posted, transmitted, distributed or disclosed in any way without *****
***** Intel's prior express written permission.
*****
***** No license under any patent, copyright or other intellectual property rights in the material is granted to  *****
***** or conferred upon you, either expressly, by implication, inducement, estoppel or otherwise. Any license     *****
***** under such intellectual property rights must be express and approved by Intel in writing.
****/

/**** Copyright Ends ****/

#include <arbb.hpp>


// for the matter of an exercise
using namespace arbb;


// step 1: figure out the function's signature
// step 4: define the function
// implementation is using ArBB types
void f(f32& result, f32 a, f32 b)
{
  // add two scalar values using ArBB
  result = a + b;
}


// step 0: main function with typical code skeleton for exception handling
int main(int /*argc*/, char* /*argv*/[])
{
  int ret_code;

  try {
    // step 2: prepare the inputs and outputs
    // prepare some scalar variables ("Hello World")
    f32 x = 25.0f, y = 7.0f, z;

    // step 3: invoke the function using call
    // compile and execute the function
    call(f)(z, x, y);

    // print the result by using the value() function (ArBB space -> C/C++ space)
    std::cout << "f(" << value(x) << ", " << value(y) << ") = " << value(z) << '\n';

    ret_code = EXIT_SUCCESS;
  }
  // ArBB exceptions can be caught as an std::exception
  catch(const std::exception& e) {
    std::cerr << "\n\nError: " << e.what() << '\n';
    ret_code = EXIT_FAILURE;
  }
  catch(...) {
    std::cerr << "\n\nError: Unknown exception caught!\n";
    ret_code = EXIT_FAILURE;
  }

  return ret_code;
}
