/****
***** Copyright 2010 Intel Corporation All Rights Reserved.
*****
***** The source code, information and material contained herein are owned by Intel Corporation or its suppliers  *****
***** or licensors, and title to such Material remains with Intel Corporation or its suppliers or licensors.      *****
***** The Material contains proprietary information of Intel or its suppliers and licensors. The Material is      *****
***** protected by worldwide copyright laws and treaty provisions. No part of the Material may be used, copied,   *****
***** reproduced, modified, published, uploaded, posted, transmitted, distributed or disclosed in any way without *****
***** Intel's prior express written permission.
*****
***** No license under any patent, copyright or other intellectual property rights in the material is granted to  *****
***** or conferred upon you, either expressly, by implication, inducement, estoppel or otherwise. Any license     *****
***** under such intellectual property rights must be express and approved by Intel in writing.
****/

/**** Copyright Ends ****/

#include <arbb.hpp>


using namespace arbb;


// generalized signature (function template)
template<typename T>
void function(T& result, const T& a, const T& b)
{
  result = a + b;
}


int main(int /*argc*/, char* /*argv*/[])
{
  int ret_code;

  try {
    const std::size_t N = 4;
    typedef array<f32, N> T;

    const T::value_type x = 1.0f;
    const T::value_type y = 2.0f;

    const T a = make_array<N>(x);
    const T b = make_array<N>(y);
    T result;

    const std::size_t ncalls = 10000;
    double time = 0;

    { // begin profiling section using scoped_timer
      scoped_timer timer(time);
      for (std::size_t i = 0; i != ncalls; ++i) {
        call(function<T>)(result, a, b);
      }
    } // end profiling section using scoped_timer

    // print the result
    std::cout << "Result: [ ";
    for (std::size_t i = 0; i != N; ++i) {
      std::cout << value(result[0]) << ' ';
    }
    std::cout << "]\n";
    std::cout << "Time  : " << time << " ms\n";

    ret_code = EXIT_SUCCESS;
  }
  catch(const std::exception& e) {
    std::cerr << "\n\nError: " << e.what() << '\n';
    ret_code = EXIT_FAILURE;
  }
  catch(...) {
    std::cerr << "\n\nError: Unknown exception caught!\n";
    ret_code = EXIT_FAILURE;
  }

  return ret_code;
}
