/****
***** Copyright 2010 Intel Corporation All Rights Reserved.
*****
***** The source code, information and material contained herein are owned by Intel Corporation or its suppliers  *****
***** or licensors, and title to such Material remains with Intel Corporation or its suppliers or licensors.      *****
***** The Material contains proprietary information of Intel or its suppliers and licensors. The Material is      *****
***** protected by worldwide copyright laws and treaty provisions. No part of the Material may be used, copied,   *****
***** reproduced, modified, published, uploaded, posted, transmitted, distributed or disclosed in any way without *****
***** Intel's prior express written permission.
*****
***** No license under any patent, copyright or other intellectual property rights in the material is granted to  *****
***** or conferred upon you, either expressly, by implication, inducement, estoppel or otherwise. Any license     *****
***** under such intellectual property rights must be express and approved by Intel in writing.
****/

/**** Copyright Ends ****/

#include <arbb.hpp>


using namespace arbb;


template<typename T>
void squared_differences(dense<T>& result, const dense<T>& a, const dense<T>& b)
{
  result = (a - b) * (a - b);
}


int main(int /*argc*/, char* /*argv*/[])
{
  int ret_code;

  try {
    typedef f32 T; // type of the inputs
    typedef f64 R; // type of the result
    // corresponding built-in type(s)
    typedef uncaptured<T>::type UT;

    const std::size_t size = 1024;
    dense<T> a(size), b(size), c;

    // make container data available via range
    range<T> ra = a.write_only_range();
    range<T> rb = b.write_only_range();

    const double scale = 1.0 / size;
    for (std::size_t i = 0; i != size; ++i) {
      ra[i] = -static_cast<UT>(scale * i);
      rb[i] = static_cast<UT>(scale * (size - i - 1));
    }

    call(squared_differences<T>)(c, a, b);

    ret_code = EXIT_SUCCESS;
  }
  catch(const std::exception& e) {
    std::cerr << "\n\nError: " << e.what() << '\n';
    ret_code = EXIT_FAILURE;
  }
  catch(...) {
    std::cerr << "\n\nError: Unknown exception caught!\n";
    ret_code = EXIT_FAILURE;
  }

  return ret_code;
}
