/****
***** Copyright 2010 Intel Corporation All Rights Reserved.
*****
***** The source code, information and material contained herein are owned by Intel Corporation or its suppliers  *****
***** or licensors, and title to such Material remains with Intel Corporation or its suppliers or licensors.      *****
***** The Material contains proprietary information of Intel or its suppliers and licensors. The Material is      *****
***** protected by worldwide copyright laws and treaty provisions. No part of the Material may be used, copied,   *****
***** reproduced, modified, published, uploaded, posted, transmitted, distributed or disclosed in any way without *****
***** Intel's prior express written permission.
*****
***** No license under any patent, copyright or other intellectual property rights in the material is granted to  *****
***** or conferred upon you, either expressly, by implication, inducement, estoppel or otherwise. Any license     *****
***** under such intellectual property rights must be express and approved by Intel in writing.
****/

/**** Copyright Ends ****/

#ifndef FIT_HPP
#define FIT_HPP

#include <algorithm>
#include <limits>


template<typename T, bool B> struct negate_if {
  static T negate(const T& t, const T&) { return -t; }
};
template<typename T> struct negate_if<T, false> {
  static T negate(const T&, const T& if_false) { return if_false; }
};

template<bool B, typename T>
T negate(const T& t, const T& if_false) {
  return negate_if<T, B>::negate(t, if_false);
}


template<typename T>
void fit(double src_min, double src_max,
  double dst_min_t, double dst_max_t,
  double dst_min_f, double dst_max_f,
  double& scale, double& shift)
{
  const double maxx = static_cast<double>(std::numeric_limits<T>::max());
  const double dst_max = std::min(maxx, std::numeric_limits<T>::is_integer ? dst_max_t : dst_max_f);

  const double minx = negate<std::numeric_limits<T>::is_signed>(dst_max, 0.0);
  const double dst_min = std::max(minx, std::numeric_limits<T>::is_integer ? dst_min_t : dst_min_f);

  const double w = src_max - src_min;
  scale = dst_max / w - dst_min / w; // avoid overflows
  shift = dst_min - scale * src_min;
}

#endif // FIT_HPP
